package lcsb.mapviewer.model.map;

import java.io.Serializable;
import java.util.Collection;
import java.util.List;

import lcsb.mapviewer.model.map.model.Model;

/**
 * Interface that describes bio entity on the map (like Protein or Reaction).
 * 
 * @author Piotr Gawron
 * 
 */
public interface BioEntity extends Serializable {
  /**
   * Returns list of {@link MiriamData annotations} for the object.
   *
   * @return list of {@link MiriamData annotations} for the object
   */
  Collection<MiriamData> getMiriamData();

  /**
   * Adds miriam annotations to the element.
   *
   * @param miriamData
   *          set of miriam annotations to be added
   */
  void addMiriamData(Collection<MiriamData> miriamData);

  /**
   * Adds miriam annotation to the element.
   *
   * @param md
   *          miriam annotation to be added
   */
  void addMiriamData(MiriamData md);

  /**
   * Returns the name of the object.
   *
   * @return the name of the object
   */
  String getName();

  /**
   * Returns notes about the object.
   *
   * @return notes about the object
   */
  String getNotes();

  /**
   * Sets notes about the object.
   *
   * @param notes
   *          new notes
   */
  void setNotes(String notes);

  /**
   * Returns the symbol of the element.
   *
   * @return the symbol of the element
   */
  String getSymbol();

  /**
   * Sets symbol of the element.
   *
   * @param symbol
   *          new symbol
   */
  void setSymbol(String symbol);

  /**
   * Get list of synonyms.
   *
   * @return list of synonyms
   */
  List<String> getSynonyms();

  /**
   * Sets list of synonyms to the element.
   *
   * @param synonyms
   *          new list
   */
  void setSynonyms(List<String> synonyms);

  /**
   * Returns the abbreviation.
   *
   * @return the abbreviation
   */
  String getAbbreviation();

  /**
   * Sets abbreviation.
   *
   * @param abbreviation
   *          new abbreviation
   */
  void setAbbreviation(String abbreviation);

  /**
   * Sets formula.
   *
   * @param formula
   *          new formula
   */
  void setFormula(String formula);

  /**
   * Returns the formula.
   *
   * @return the formula
   */
  String getFormula();

  /**
   * Sets the name to the object.
   *
   * @param name
   *          name of the object
   *
   */
  void setName(String name);

  /**
   * Returns database identifier of the object.
   *
   * @return database identifier of the object
   */
  int getId();

  /**
   * Return human readable {@link String} representing class.
   *
   * @return human readable {@link String} representing class
   */
  String getStringType();

  /**
   * Sets semantic zoom level visibility.
   *
   * @param zoomLevelVisibility
   *          semantic zoom level visibility
   */
  void setVisibilityLevel(String zoomLevelVisibility);

  /**
   * Sets semantic zoom level visibility.
   *
   * @param zoomLevelVisibility
   *          semantic zoom level visibility
   */
  void setVisibilityLevel(Integer zoomLevelVisibility);

  /**
   * Returns semantic zoom level visibility.
   * 
   * @return semantic zoom level visibility
   */
  String getVisibilityLevel();

  /**
   * Returns the {@link Model} where BioEntity is located.
   *
   * @return the model
   */
  Model getModel();

  String getElementId();

}
