package lcsb.mapviewer.model.map.graph;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinTable;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import org.hibernate.annotations.Cascade;
import org.hibernate.annotations.CascadeType;
import org.hibernate.annotations.IndexColumn;

import lcsb.mapviewer.model.map.MiriamData;
import lcsb.mapviewer.model.map.species.Element;

/**
 * Entity representing data mining entry.
 * 
 * @author Piotr Gawron
 * 
 */
@Entity
@Table(name = "datamining_table")
public class DataMining implements Serializable {

	/**
	 * 
	 */
	private static final long	serialVersionUID		 = 1L;

	/**
	 * Unique database identifier.
	 */
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name = "idDb", unique = true, nullable = false)
	private Integer						id;

	/**
	 * For which element this entry was created.
	 */
	@ManyToOne(optional = false)
	private Element							element;

	/**
	 * List of the suggested connection.
	 */
	@Cascade({ CascadeType.ALL })
	@OneToMany(fetch = FetchType.EAGER)
	@IndexColumn(name = "idx")
	@JoinTable(name = "datamining_connections")
	private List<MiriamData>	suggestedConnections = new ArrayList<>();

	/**
	 * Short description of the connection.
	 */
	@Column(columnDefinition = "TEXT")
	private String						description;

	/**
	 * Where the data comes from.
	 */
	private DataMiningType		type;

	/**
	 * List of reference links for this entry.
	 */
	@Cascade({ CascadeType.ALL })
	@OneToMany(fetch = FetchType.EAGER)
	@IndexColumn(name = "idx")
	@JoinTable(name = "datamining_references")
	private List<MiriamData>	references					 = new ArrayList<>();

	/**
	 * Adds reference.
	 * 
	 * @param md
	 *          new reference link
	 * @see #references
	 */
	public void addReference(MiriamData md) {
		references.add(md);
	}

	/**
	 * @return the references
	 * @see #references
	 */
	public List<MiriamData> getReferences() {
		return references;
	}

	/**
	 * @param references
	 *          the references to set
	 * @see #references
	 */
	public void setReferences(List<MiriamData> references) {
		this.references = references;
	}

	/**
	 * @return the type
	 * @see #type
	 */
	public DataMiningType getType() {
		return type;
	}

	/**
	 * @param type
	 *          the type to set
	 * @see #type
	 */
	public void setType(DataMiningType type) {
		this.type = type;
	}

	/**
	 * @return the description
	 * @see #description
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * @param description
	 *          the description to set
	 * @see #description
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * @return the id
	 * @see #id
	 */
	public Integer getId() {
		return id;
	}

	/**
	 * @param id
	 *          the id to set
	 * @see #id
	 */
	public void setId(Integer id) {
		this.id = id;
	}

	/**
	 * @return the element
	 * @see #element
	 */
	public Element getElement() {
		return element;
	}

	/**
	 * @param element
	 *          the element to set
	 * @see #element
	 */
	public void setElement(Element element) {
		this.element = element;
	}

	/**
	 * @return the suggestedConnections
	 * @see #suggestedConnections
	 */
	public List<MiriamData> getSuggestedConnections() {
		return suggestedConnections;
	}

	/**
	 * @param suggestedConnections
	 *          the suggestedConnections to set
	 * @see #suggestedConnections
	 */
	public void setSuggestedConnections(List<MiriamData> suggestedConnections) {
		this.suggestedConnections = suggestedConnections;
	}

	/**
	 * @param suggestedConnection
	 *          the suggestedConnection to add
	 * @see #suggestedConnections
	 */
	public void addSuggestedConnection(MiriamData suggestedConnection) {
		this.suggestedConnections.add(suggestedConnection);
	}

	/**
	 * Adds references to {@link #references}.
	 * 
	 * @param ref
	 *          references to add
	 */
	public void addReferences(List<MiriamData> ref) {
		references.addAll(ref);
	}

}
