package lcsb.mapviewer.model.map.graph;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.model.ModelData;

/**
 * This class describes single data mining set for a model (for instance set of
 * predicted text mining connections).
 * 
 * @author Piotr Gawron
 * 
 */
@Entity
@Table(name = "data_mining_set_table")
public class DataMiningSet implements Serializable {

	/**
	 * 
	 */
	private static final long	serialVersionUID	= 1L;

	/**
	 * Unique database identifier.
	 */
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name = "idDb", unique = true, nullable = false)
	private int								id;

	/**
	 * {@link ModelData Model} where the {@link DataMiningSet} belongs to.
	 */
	@ManyToOne(fetch = FetchType.LAZY)
	private ModelData					model;

	/**
	 * Name of the data mining set.
	 */
	private String						name;

	/**
	 * Type of the data mining set.
	 */
	private DataMiningType		type;

	/**
	 * Description of the data mining set.
	 */
	private String						description;

	/**
	 * Link to the source of the data mining set.
	 */
	private String						source;

	/**
	 * Original data of this set.
	 */
	private byte[]						inputData;

	/**
	 * Default constructor.
	 */
	public DataMiningSet() {

	}

	/**
	 * Default constructor.
	 * 
	 * @param original
	 *          original object from which this one will be initialized
	 */
	public DataMiningSet(DataMiningSet original) {
		this.name = original.getName();

		this.type = original.getType();
		this.description = original.getDescription();
		this.source = original.getSource();

		if (original.getInputData() != null) {
			this.inputData = original.getInputData().clone();
		}

	}

	/**
	 * @return the name
	 * @see #name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name
	 *          the name to set
	 * @see #name
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the inputData
	 * @see #inputData
	 */
	public byte[] getInputData() {
		return inputData;
	}

	/**
	 * @param inputData
	 *          the inputData to set
	 * @see #inputData
	 */
	public void setInputData(byte[] inputData) {
		this.inputData = inputData;
	}

	/**
	 * @return the model
	 * @see #model
	 */
	public ModelData getModel() {
		return model;
	}

	/**
	 * @param model
	 *          the model to set
	 * @see #model
	 */
	public void setModel(ModelData model) {
		this.model = model;
	}

	/**
	 * @param model
	 *          the model to set
	 * @see #model
	 */
	public void setModel(Model model) {
		this.model = model.getModelData();
	}

	/**
	 * @return the id
	 * @see #id
	 */
	public int getId() {
		return id;
	}

	/**
	 * @param id
	 *          the id to set
	 * @see #id
	 */
	public void setId(int id) {
		this.id = id;
	}

	/**
	 * @return the description
	 * @see #description
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * @param description
	 *          the description to set
	 * @see #description
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * @return the source
	 * @see #source
	 */
	public String getSource() {
		return source;
	}

	/**
	 * @param source
	 *          the source to set
	 * @see #source
	 */
	public void setSource(String source) {
		this.source = source;
	}

	/**
	 * @return the type
	 * @see #type
	 */
	public DataMiningType getType() {
		return type;
	}

	/**
	 * @param type
	 *          the type to set
	 * @see #type
	 */
	public void setType(DataMiningType type) {
		this.type = type;
	}
}
