package lcsb.mapviewer.model.map.species;

import javax.persistence.DiscriminatorValue;
import javax.persistence.Entity;

import lcsb.mapviewer.common.exception.NotImplementedException;

/**
 * Entity representing drug element on the map.
 * 
 * @author Piotr Gawron
 *
 */
@Entity
@DiscriminatorValue("DRUG_ALIAS")
public class Drug extends Species {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Empty constructor required by hibernate.
	 */
	Drug() {
	}

	/**
	 * Constructor that creates a copy of the element given in the parameter.
	 * 
	 * @param original
	 *          original object that will be used for creating copy
	 */
	public Drug(Drug original) {
		super(original);
	}

	/**
	 * Default constructor.
	 * 
	 * @param elementId
	 *          uniqe (within model) element identifier
	 */
	public Drug(String elementId) {
		setElementId(elementId);
	}

	@Override
	public Drug copy() {
		if (this.getClass() == Drug.class) {
			return new Drug(this);
		} else {
			throw new NotImplementedException("Method copy() should be overriden in class " + this.getClass());
		}
	}

	@Override
	public String getStringType() {
		return "Drug";
	}


}
