package lcsb.mapviewer.model.map.species;

import javax.persistence.DiscriminatorValue;
import javax.persistence.Entity;

import lcsb.mapviewer.common.exception.NotImplementedException;

/**
 * Entity representing ion channel protein element on the map.
 * 
 * @author Piotr Gawron
 *
 */
@Entity
@DiscriminatorValue("ION_CHANNEL_PROTEIN_ALIAS")
public class IonChannelProtein extends Protein {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Empty constructor required by hibernate.
	 */
	IonChannelProtein() {
	}

	/**
	 * Constructor that creates a copy of the element given in the parameter.
	 * 
	 * @param original
	 *          original object that will be used for creating copy
	 */
	public IonChannelProtein(IonChannelProtein original) {
		super(original);
	}

	/**
	 * Default constructor.
	 * 
	 * @param elementId
	 *          uniqe (within model) element identifier
	 */
	public IonChannelProtein(String elementId) {
		setElementId(elementId);
	}

	@Override
	public IonChannelProtein copy() {
		if (this.getClass() == IonChannelProtein.class) {
			return new IonChannelProtein(this);
		} else {
			throw new NotImplementedException("Method copy() should be overriden in class " + this.getClass());
		}
	}

}
