package lcsb.mapviewer.model.map.species.field;

import lcsb.mapviewer.model.map.species.AntisenseRna;

/**
 * Enum that enlists types of available modification regions in
 * {@link AntisenseRna}.
 * 
 * @author Piotr Gawron
 * 
 */
public enum AntisenseRnaRegionType {

	/**
	 * Modification site.
	 */
	MODIFICATION_SITE("Modification Site"), //

	/**
	 * Coding region.
	 */
	CODING_REGION("CodingRegion"), //

	/**
	 * Protein binding domain.
	 */
	PROTEIN_BINDING_DOMAIN("proteinBindingDomain");

	/**
	 * Name of the type.
	 */
	private String name;

	/**
	 * Default constructor.
	 * 
	 * @param name
	 *          name of the type
	 */
	AntisenseRnaRegionType(String name) {
		this.name = name;
	}

	/**
	 * @return the name
	 * @see #name
	 */
	public String getName() {
		return name;
	}

	/**
	 * Returns {@link AntisenseRnaRegionType} identified by name.
	 * 
	 * @param name
	 *          name of the region
	 * @return {@link AntisenseRnaRegionType} identified by name
	 */
	public static AntisenseRnaRegionType getTypeByString(String name) {
		for (AntisenseRnaRegionType type : values()) {
			if (type.getName().equalsIgnoreCase(name)) {
				return type;
			}
		}
		return null;
	}
}