package lcsb.mapviewer.model.map.species.field;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.apache.log4j.Logger;

import lcsb.mapviewer.common.exception.NotImplementedException;
import lcsb.mapviewer.model.map.species.Species;

/**
 * This class represent modification residue in protein and gene. However, it is
 * sometimes also used for storing information about AntisenseRna/Rna regions...
 * (due to CellDesigner xml strange structure).
 * 
 * @author Piotr Gawron
 * 
 */
@Entity
@Table(name = "modification_residue_table")
@org.hibernate.annotations.GenericGenerator(name = "test-increment-strategy", strategy = "increment")
public class ModificationResidue implements Serializable, ElementModification {

	/**
	 * 
	 */
	private static final long	serialVersionUID			= 1L;

	/**
	 * Default class logger.
	 */
	@SuppressWarnings("unused")
	private static Logger			logger								= Logger.getLogger(ModificationResidue.class.getName());

	/**
	 * Unique identifier in the database.
	 */
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name = "iddb", unique = true, nullable = false)
	private int								id;

	/**
	 * Identifier of the modification. Must be unique in single map model.
	 */
	private String						idModificationResidue	= "";

	/**
	 * Name of the modification.
	 */
	private String						name									= "";

	/**
	 * Some strange param in CellDesigner. No idea what is it for.
	 */
	private String						side									= "";

	/**
	 * State in which this modification is.
	 */
	private ModificationState	state									= null;

	/**
	 * Where this modification is located (on which side of the border).
	 */
	private Double						angle									= null;

	/**
	 * How big is this modification (used only for some types of the
	 * modification).
	 */
	private Double						size									= null;

	/**
	 * Species to which this modification belong to.
	 */
	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "idSpeciesDb", nullable = false)
	private Species						species;

	/**
	 * Default constructor.
	 */
	public ModificationResidue() {
	}

	/**
	 * Constructor that initaize object with the data taken from the parameter.
	 * 
	 * @param mr
	 *          original object from which data is taken
	 */
	public ModificationResidue(ModificationResidue mr) {
		this.idModificationResidue = mr.idModificationResidue;
		this.name = mr.name;
		this.angle = mr.angle;
		this.size = mr.size;
		this.side = mr.side;
		this.state = mr.state;
	}

	@Override
	public String toString() {
		String result = getIdModificationResidue() + "," + getName() + "," + getState() + "," + getAngle() + "," + getSize() + "," + getSide() + ",";
		return result;
	}

	/**
	 * Creates copy of the object.
	 * 
	 * @return copy of the object.
	 */
	public ModificationResidue copy() {
		if (this.getClass() == ModificationResidue.class) {
			return new ModificationResidue(this);
		} else {
			throw new NotImplementedException("Method copy() should be overriden in class " + this.getClass());
		}
	}

	/**
	 * @return the idModificationResidue
	 * @see #id
	 */
	public int getId() {
		return id;
	}

	/**
	 * @param id
	 *          the idModificationResidue to set
	 * @see #id
	 */
	public void setId(int id) {
		this.id = id;
	}

	/**
	 * @return the id
	 * @see #idModificationResidue
	 */
	public String getIdModificationResidue() {
		return idModificationResidue;
	}

	/**
	 * @param idModificationResidue
	 *          the id to set
	 * @see #idModificationResidue
	 */
	public void setIdModificationResidue(String idModificationResidue) {
		this.idModificationResidue = idModificationResidue;
	}

	/**
	 * @return the name
	 * @see #name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name
	 *          the name to set
	 * @see #name
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the side
	 * @see #side
	 */
	public String getSide() {
		return side;
	}

	/**
	 * @param side
	 *          the side to set
	 * @see #side
	 */
	public void setSide(String side) {
		this.side = side;
	}

	/**
	 * @return the state
	 * @see #state
	 */
	public ModificationState getState() {
		return state;
	}

	/**
	 * @param state
	 *          the state to set
	 * @see #state
	 */
	public void setState(ModificationState state) {
		this.state = state;
	}

	/**
	 * @return the angle
	 * @see #angle
	 */
	public Double getAngle() {
		return angle;
	}

	/**
	 * @param angle
	 *          the angle to set
	 * @see #angle
	 */
	public void setAngle(Double angle) {
		this.angle = angle;
	}

	/**
	 * @return the size
	 * @see #size
	 */
	public Double getSize() {
		return size;
	}

	/**
	 * @param size
	 *          the size to set
	 * @see #size
	 */
	public void setSize(Double size) {
		this.size = size;
	}

	/**
	 * @return the species
	 * @see #species
	 */
	public Species getSpecies() {
		return species;
	}

	/**
	 * @param species
	 *          the species to set
	 * @see #species
	 */
	public void setSpecies(Species species) {
		this.species = species;
	}

}
