package lcsb.mapviewer.model.map;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.fail;

import org.apache.commons.lang3.SerializationUtils;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import lcsb.mapviewer.common.exception.NotImplementedException;
import lcsb.mapviewer.model.map.SearchIndex;
import lcsb.mapviewer.model.map.species.Element;
import lcsb.mapviewer.model.map.species.Gene;

public class SearchIndexTest {

	@Before
	public void setUp() throws Exception {
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void testSerialization() {
		try {
			SerializationUtils.serialize(new SearchIndex());
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testConstructor1() {
		try {
			SearchIndex index = new SearchIndex("str", 2);
			assertEquals("str", index.getValue());
			assertEquals((Integer) 2, index.getWeight());
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testConstructor2() {
		try {
			SearchIndex index = new SearchIndex("str");
			assertEquals("str", index.getValue());
			assertEquals((Integer) 1, index.getWeight());
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testConstructor3() {
		try {
			SearchIndex index = new SearchIndex("str");
			SearchIndex index2 = new SearchIndex(index);
			assertNotNull(index2);
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testCopy() {
		try {
			SearchIndex aRna = new SearchIndex().copy();
			assertNotNull(aRna);
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testCopyInvalid() {
		try {
			new SearchIndex() {
				/**
				 * 
				 */
				private static final long serialVersionUID = 1L;
			}.copy();
			fail("Exception expected");
		} catch (NotImplementedException e) {
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testGetters() {
		try {
			int id = 54;
			int weight = 8;
			String value = "val5";
			Element element = new Gene("gene_id");

			SearchIndex index = new SearchIndex();
			index.setId(id);
			assertEquals(id, index.getId());

			index.setValue(value);
			assertEquals(value, index.getValue());

			index.setSource(element);
			assertEquals(element, index.getSource());

			index.setWeight(weight);
			assertEquals((Integer) weight, index.getWeight());

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

}
