package lcsb.mapviewer.model.map.graph;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import lcsb.mapviewer.common.exception.NotImplementedException;

public class DataMiningSetComparatorTest {

	DataMiningSetComparator comparator = new DataMiningSetComparator();

	@Before
	public void setUp() throws Exception {
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void testCompareEmpty() throws Exception {
		try {
			DataMiningSet obj1 = new DataMiningSet();
			DataMiningSet obj2 = new DataMiningSet();

			assertTrue(comparator.compare(obj1, obj2) == 0);
			assertTrue(comparator.compare(obj2, obj1) == 0);
			assertTrue(comparator.compare(obj1, obj1) == 0);
			assertTrue(comparator.compare(obj2, obj2) == 0);

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testCompareNull() throws Exception {
		try {
			DataMiningSet obj1 = new DataMiningSet();
			DataMiningSet obj2 = new DataMiningSet();

			assertTrue(comparator.compare(null, null) == 0);
			assertTrue(comparator.compare(obj2, null) != 0);
			assertTrue(comparator.compare(null, obj1) != 0);

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testCompareInvalid() throws Exception {
		try {
			DataMiningSet obj1 = new DataMiningSet();
			DataMiningSet obj2 = new DataMiningSet();

			assertTrue(comparator.compare(obj2, new DataMiningSet() {
				private static final long serialVersionUID = 1L;
			}) != 0);
			assertTrue(comparator.compare(null, obj1) != 0);

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testCompareInvalid2() throws Exception {
		try {
			
			class Tmp extends DataMiningSet {
				private static final long serialVersionUID = 1L;
			};
			DataMiningSet obj1 = new Tmp();
			DataMiningSet obj2 = new Tmp();

			comparator.compare(obj2, obj1);

			fail("Exception expeced");
		} catch (NotImplementedException e) {
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testCompareName() throws Exception {
		try {
			DataMiningSet obj1 = new DataMiningSet();
			DataMiningSet obj2 = new DataMiningSet();
			obj1.setName("test");

			assertTrue(comparator.compare(obj1, obj2) != 0);
			assertTrue(comparator.compare(obj2, obj1) != 0);

			obj2.setName("test");

			assertTrue(comparator.compare(obj1, obj2) == 0);
			assertTrue(comparator.compare(obj2, obj1) == 0);

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

}
