package lcsb.mapviewer.model.map.model;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import lcsb.mapviewer.model.map.compartment.Compartment;
import lcsb.mapviewer.model.map.layout.graphics.Layer;
import lcsb.mapviewer.model.map.reaction.Reaction;
import lcsb.mapviewer.model.map.species.GenericProtein;
import lcsb.mapviewer.model.map.species.Species;

public class ModelSubmodelConnectionComparatorTest {

	ModelSubmodelConnectionComparator comparator = new ModelSubmodelConnectionComparator();

	@Before
	public void setUp() throws Exception {
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void testEquals() throws Exception {
		try {
			ModelSubmodelConnection connectionA = createConnection();
			ModelSubmodelConnection connectionB = createConnection();

			assertEquals(0, comparator.compare(new ModelSubmodelConnection(), new ModelSubmodelConnection()));
			assertEquals(0, comparator.compare(connectionA, connectionB));
			assertEquals(0, comparator.compare(connectionA, connectionA));
			assertEquals(0, comparator.compare(null, null));

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testDifferent() throws Exception {
		try {
			ModelSubmodelConnection connectionA = createConnection();
			ModelSubmodelConnection connectionB = createConnection();

			assertTrue(comparator.compare(null, connectionB) != 0);
			assertTrue(comparator.compare(connectionA, null) != 0);

			connectionA.getParentModel().setNotes("QQWECS");
			assertTrue(comparator.compare(connectionA, connectionB) != 0);
			assertTrue(comparator.compare(connectionB, connectionA) != 0);

			connectionA = createConnection();
			connectionB = createConnection();

			connectionA.setParentModel((ModelData) null);

			assertTrue(comparator.compare(connectionA, connectionB) != 0);
			assertTrue(comparator.compare(connectionB, connectionA) != 0);

			connectionA = createConnection();
			connectionB = createConnection();

			connectionA.setType(SubmodelType.UNKNOWN);

			assertTrue(comparator.compare(connectionA, connectionB) != 0);
			assertTrue(comparator.compare(connectionB, connectionA) != 0);

			assertTrue(comparator.compare(connectionB, new ModelSubmodelConnection() {
				private static final long serialVersionUID = 1L;
			}) != 0);

		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	private Model getModel() {
		Model model = new ModelFullIndexed(null);

		model.setNotes("Some description");

		Species species = new GenericProtein("a_id");
		species.setName("ad");
		model.addElement(species);

		model.addElement(new Compartment("default"));

		Layer layer = new Layer();
		layer.setName("layer name");
		model.addLayer(layer);

		model.addReaction(new Reaction());
		return model;
	}

	private ModelSubmodelConnection createConnection() {
		ModelSubmodelConnection result = new ModelSubmodelConnection(getModel(), SubmodelType.DOWNSTREAM_TARGETS);
		result.setName("name A");
		result.setParentModel(getModel());
		return result;
	}

}
