package lcsb.mapviewer.model.map.species;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import lcsb.mapviewer.common.exception.NotImplementedException;
import lcsb.mapviewer.model.map.species.field.ModificationResidue;
import lcsb.mapviewer.model.map.species.field.ModificationState;

public class GeneComparatorTest {

	GeneComparator comparator = new GeneComparator();

	@Before
	public void setUp() throws Exception {
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void testEquals() {
		try {
			Gene gene1 = createGene();
			Gene gene2 = createGene();

			assertEquals(0, comparator.compare(gene1, gene1));

			assertEquals(0, comparator.compare(gene1, gene2));
			assertEquals(0, comparator.compare(gene2, gene1));

		} catch (Exception e) {
			e.printStackTrace();
			fail("Unknowne exception occurred");
		}
	}

	@Test
	public void testDifferent() {
		try {
			Gene gene1 = createGene();
			Gene gene2 = createGene();
			gene1.getModificationResidues().get(0).setName("bla");
			assertTrue(comparator.compare(gene1, gene2) != 0);
			assertTrue(comparator.compare(gene2, gene1) != 0);

			gene1 = createGene();
			gene2 = createGene();
			gene1.getModificationResidues().clear();
			assertTrue(comparator.compare(gene1, gene2) != 0);
			assertTrue(comparator.compare(gene2, gene1) != 0);

			gene1 = createGene();
			gene2 = createGene();
			assertTrue(comparator.compare(null, gene2) != 0);
			assertTrue(comparator.compare(gene2, null) != 0);
			assertTrue(comparator.compare(null, null) == 0);

			Gene gene = createGene();
			gene.setName("n");
			assertTrue(comparator.compare(gene, gene2) != 0);

			assertTrue(comparator.compare(gene, Mockito.mock(Gene.class)) != 0);

		} catch (Exception e) {
			e.printStackTrace();
			fail("Unknowne exception occurred");
		}
	}

	public Gene createGene() {
		Gene result = new Gene();
		result.setHypothetical(true);

		ModificationResidue residue = new ModificationResidue();
		result.addModificationResidue(residue);

		residue.setIdModificationResidue("a");
		residue.setName("name");
		residue.setAngle(1.0);
		residue.setSize(2.0);
		residue.setSide("23");
		residue.setState(ModificationState.ACETYLATED);
		return result;
	}

	@Test
	public void testInvalid() {
		try {
			Gene object = Mockito.mock(Gene.class);

			comparator.compare(object, object);

			fail("Exception expected");
		} catch (NotImplementedException e) {

		} catch (Exception e) {
			e.printStackTrace();
			fail("Unknowne exception occurred");
		}
	}

}
