package lcsb.mapviewer.model.map.species;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import lcsb.mapviewer.common.exception.NotImplementedException;
import lcsb.mapviewer.model.map.species.field.ModificationState;
import lcsb.mapviewer.model.map.species.field.RnaRegion;

public class RnaComparatorTest {

	RnaComparator comparator = new RnaComparator();

	@Before
	public void setUp() throws Exception {
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void testEquals() {
		try {
			Rna aRna1 = createRna();
			Rna aRna2 = createRna();

			assertEquals(0, comparator.compare(aRna1, aRna1));

			assertEquals(0, comparator.compare(aRna1, aRna2));
			assertEquals(0, comparator.compare(aRna2, aRna1));

		} catch (Exception e) {
			e.printStackTrace();
			fail("Unknowne exception occurred");
		}
	}

	@Test
	public void testDifferent() {
		try {
			Rna aRna1 = createRna();
			Rna aRna2 = createRna();
			aRna1.getRegions().get(0).setState(ModificationState.ACETYLATED);
			assertTrue(comparator.compare(aRna1, aRna2) != 0);
			assertTrue(comparator.compare(aRna2, aRna1) != 0);

			aRna1 = createRna();
			aRna2 = createRna();
			aRna1.getRegions().clear();
			assertTrue(comparator.compare(aRna1, aRna2) != 0);
			assertTrue(comparator.compare(aRna2, aRna1) != 0);

			aRna1 = createRna();
			aRna2 = createRna();
			assertTrue(comparator.compare(null, aRna2) != 0);
			assertTrue(comparator.compare(aRna2, null) != 0);
			assertTrue(comparator.compare(null, null) == 0);

			Rna rna = createRna();
			rna.setName("n");
			assertTrue(comparator.compare(rna, aRna2) != 0);

			assertTrue(comparator.compare(rna, Mockito.mock(Rna.class)) != 0);

		} catch (Exception e) {
			e.printStackTrace();
			fail("Unknowne exception occurred");
		}
	}

	public Rna createRna() {
		Rna result = new Rna();
		result.setHypothetical(true);

		RnaRegion region1 = new RnaRegion();
		result.addRegion(region1);
		region1.setIdRnaRegion("a");
		region1.setState(ModificationState.DONT_CARE);
		region1.setPos("1");
		region1.setSize("2");
		region1.setType("type");
		return result;
	}

	@Test
	public void testInvalid() {
		try {
			Rna object = Mockito.mock(Rna.class);

			comparator.compare(object, object);

			fail("Exception expected");
		} catch (NotImplementedException e) {

		} catch (Exception e) {
			e.printStackTrace();
			fail("Unknowne exception occurred");
		}
	}

}
