package lcsb.mapviewer.model.map.species.field;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.fail;

import org.apache.commons.lang3.SerializationUtils;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import lcsb.mapviewer.common.Configuration;
import lcsb.mapviewer.common.exception.InvalidArgumentException;
import lcsb.mapviewer.model.map.species.Rna;

public class RnaRegionTest {

	@Before
	public void setUp() throws Exception {
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void testSerialization() {
		try {
			SerializationUtils.serialize(new RnaRegion());
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testConstructor() {
		try {
			RnaRegion region = new RnaRegion(new RnaRegion());
			assertNotNull(region);
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testSetSize() {
		try {
			RnaRegion region = new RnaRegion();
			try {
				region.setSize("as");
				fail("Exception expected");
			} catch (InvalidArgumentException e) {
			}
			region.setSize("0.0");
			assertEquals(0.0, region.getSize(), Configuration.EPSILON);
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testSetPos() {
		try {
			RnaRegion region = new RnaRegion();
			try {
				region.setPos("as");
				fail("Exception expected");
			} catch (InvalidArgumentException e) {
			}
			region.setPos("1.0");
			assertEquals(1.0, region.getPos(), Configuration.EPSILON);
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testUpdate() {
		try {
			RnaRegion region = new RnaRegion();
			RnaRegion region2 = new RnaRegion();
			region2.setState(ModificationState.ACETYLATED);
			region2.setName("asd");
			region2.setPos(2.2);
			region.update(region2);
			assertEquals(region2.getName(), region.getName());
			assertEquals(region2.getPos(), region.getPos());
			assertEquals(region2.getState(), region.getState());
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testUpdate2() {
		try {
			RnaRegion region = new RnaRegion();
			region.setIdRnaRegion("1");
			RnaRegion region2 = new RnaRegion();
			region2.setIdRnaRegion("2");
			region.update(region2);
			fail("Exception expected");
		} catch (InvalidArgumentException e) {
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}

	@Test
	public void testGetters() {
		try {
			RnaRegion region = new RnaRegion();
			double size = 2.5;
			int id = 58;
			Rna species = new Rna("id");
			region.setSize(size);
			region.setSpecies(species);
			region.setId(id);
			assertEquals(id, region.getId());
			assertEquals(size, region.getSize(), Configuration.EPSILON);
			assertEquals(species, region.getSpecies());
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
	}
}
